import { Args } from '@oclif/core';
import chalk from 'chalk';
import { ApifyCommand } from '../../lib/apify_command.js';
import { prettyPrintBytes } from '../../lib/commands/pretty-print-bytes.js';
import { prettyPrintStatus } from '../../lib/commands/pretty-print-status.js';
import { error, simpleLog } from '../../lib/outputs.js';
import { DurationFormatter, getLoggedClientOrThrow, TimestampFormatter } from '../../lib/utils.js';
export class BuildInfoCommand extends ApifyCommand {
    async run() {
        const { buildId } = this.args;
        const apifyClient = await getLoggedClientOrThrow();
        const build = await apifyClient.build(buildId).get();
        if (!build) {
            error({ message: `Build with ID "${buildId}" was not found on your account.`, stdout: true });
            return;
        }
        // JSON output -> return the object (which is handled by oclif)
        if (this.flags.json) {
            return build;
        }
        const actor = await apifyClient.actor(build.actId).get();
        let buildTag;
        if (actor?.taggedBuilds) {
            for (const [tag, buildData] of Object.entries(actor.taggedBuilds)) {
                if (buildData.buildId === build.id) {
                    buildTag = tag;
                    break;
                }
            }
        }
        // TODO: untyped field, https://github.com/apify/apify-client-js/issues/526
        const exitCode = Reflect.get(build, 'exitCode');
        const fullActorName = actor?.username ? `${actor.username}/${actor.name}` : (actor?.name ?? 'unknown-actor');
        const versionTaggedAs = buildTag ? ` (tagged as ${chalk.yellow(buildTag)})` : '';
        const exitCodeStatus = typeof exitCode !== 'undefined' ? ` (exit code: ${chalk.gray(exitCode)})` : '';
        const message = [
            //
            `${chalk.yellow('Actor')}: ${fullActorName} (${chalk.gray(build.actId)})`,
            '',
            `${chalk.yellow('Build Information')} (ID: ${chalk.gray(build.id)})`,
            `  ${chalk.yellow('Build Number')}: ${build.buildNumber}${versionTaggedAs}`,
            `  ${chalk.yellow('Status')}: ${prettyPrintStatus(build.status)}${exitCodeStatus}`,
            `  ${chalk.yellow('Started')}: ${TimestampFormatter.display(build.startedAt)}`,
        ];
        if (build.finishedAt) {
            message.push(`  ${chalk.yellow('Finished')}: ${TimestampFormatter.display(build.finishedAt)} (took ${chalk.gray(DurationFormatter.format(build.stats?.durationMillis ?? 0))})`);
        }
        else {
            const diff = Date.now() - build.startedAt.getTime();
            message.push(`  ${chalk.yellow('Finished')}: ${chalk.gray(`Running for ${DurationFormatter.format(diff)}`)}`);
        }
        if (build.stats?.computeUnits) {
            // Platform shows 3 decimal places, so shall we
            message.push(`  ${chalk.yellow('Compute Units')}: ${build.stats.computeUnits.toFixed(3)}`);
        }
        // TODO: untyped field, https://github.com/apify/apify-client-js/issues/526
        const dockerImageSize = Reflect.get(build.stats ?? {}, 'imageSizeBytes');
        if (dockerImageSize) {
            message.push(`  ${chalk.yellow('Docker Image Size')}: ${prettyPrintBytes(dockerImageSize)}`);
        }
        message.push(`  ${chalk.yellow('Origin')}: ${build.meta.origin ?? 'UNKNOWN'}`);
        message.push('');
        const url = `https://console.apify.com/actors/${build.actId}/builds/${build.buildNumber}`;
        message.push(`${chalk.blue('View in Apify Console')}: ${url}`);
        simpleLog({ message: message.join('\n'), stdout: true });
        return undefined;
    }
}
Object.defineProperty(BuildInfoCommand, "description", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: 'Prints information about a specific build.'
});
Object.defineProperty(BuildInfoCommand, "args", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: {
        buildId: Args.string({
            required: true,
            description: 'The build ID to get information about.',
        }),
    }
});
Object.defineProperty(BuildInfoCommand, "enableJsonFlag", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: true
});
//# sourceMappingURL=info.js.map