import process from 'node:process';
import { Command, loadHelpClass } from '@oclif/core';
import { COMMANDS_WITHIN_ACTOR, LANGUAGE } from './consts.js';
import { maybeTrackTelemetry } from './telemetry.js';
import { argsToCamelCase, detectLocalActorLanguage } from './utils.js';
import { detectInstallationType } from './version_check.js';
/**
 * Adding parsing flags to oclif Command class
 */
export class ApifyCommand extends Command {
    constructor() {
        super(...arguments);
        Object.defineProperty(this, "telemetryData", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "flags", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "args", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
    }
    async init() {
        await super.init();
        const { args, flags } = await this.parse({
            flags: this.ctor.flags,
            args: this.ctor.args,
            strict: this.ctor.strict,
            enableJsonFlag: this.ctor.enableJsonFlag,
        });
        this.flags = argsToCamelCase(flags);
        this.args = args;
        // Initialize telemetry data, all attributes are tracked in the finally method
        this.telemetryData = {
            flagsUsed: Object.keys(this.flags),
        };
    }
    async finally(err) {
        const command = this.id;
        const eventData = {
            command,
            $os: this.config.platform,
            shell: this.config.shell,
            arch: this.config.arch,
            apifyCliVersion: this.config.version,
            nodeJsVersion: process.version,
            ...this.telemetryData,
            error: err ? err.message : null,
        };
        try {
            eventData.installationType = detectInstallationType();
            if (!this.telemetryData.actorLanguage && command && COMMANDS_WITHIN_ACTOR.includes(command)) {
                const { language, languageVersion } = detectLocalActorLanguage(process.cwd());
                eventData.actorLanguage = language;
                if (language === LANGUAGE.NODEJS) {
                    eventData.actorNodejsVersion = languageVersion;
                }
                else if (language === LANGUAGE.PYTHON) {
                    eventData.actorPythonVersion = languageVersion;
                }
            }
            await maybeTrackTelemetry({
                eventName: `cli_command_${command}`,
                eventData,
            });
        }
        catch {
            // Ignore errors
        }
        return super.finally(err);
    }
    async printHelp(customCommand) {
        const HelpCommand = await loadHelpClass(this.config);
        const help = new HelpCommand(this.config);
        await help.showHelp([customCommand ?? this.id]);
    }
    pluralString(amount, singular, plural) {
        return amount === 1 ? singular : plural;
    }
}
//# sourceMappingURL=apify_command.js.map